@@ -1,6 +1,6 @@
 // main.cpp
 
-// Copyright (C) 2011-2013 by Werner Lemberg.
+// Copyright (C) 2011-2014 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -70,7 +70,7 @@ typedef struct Script_Names_
 } Script_Names;
 
 #undef SCRIPT
-#define SCRIPT(s, S, d) \
+#define SCRIPT(s, S, d, h, sc1, sc2, sc3) \
           {#s, d},
 
 const Script_Names script_names[] =
@@ -182,7 +182,8 @@ show_help(bool
 #endif
 "  -c, --composites           hint glyph composites also\n"
 "  -d, --dehint               remove all hints\n"
-"  -f, --fallback-script=S    set fallback script (default: dflt)\n"
+"  -D, --default-script=S     set default OpenType script (default: latn)\n"
+"  -f, --fallback-script=S    set fallback script (default: none)\n"
 "  -G, --hinting-limit=N      switch off hinting above this PPEM value\n"
 "                             (default: %d); value 0 means no limit\n"
 "  -h, --help                 display this help and exit\n"
@@ -288,9 +289,10 @@ show_help(bool
 "for all sizes (limited by option -G, which is handled in the bytecode).\n"
 "\n");
   fprintf(handle,
-"Option -f takes a four-letter string that identifies the script\n"
-"to be used as a fallback for glyphs that have character codes\n"
-"outside of known script ranges.  Possible values are\n"
+"Options -f and -D take a four-letter string that identifies a script.\n"
+"Option -f sets the script used as a fallback for glyphs that have\n"
+"character codes outside of known script ranges.  Option -D sets the\n"
+"default script for handling OpenType features.  Possible values are\n"
 "\n");
   const Script_Names* sn = script_names;
   for(;;)
@@ -308,10 +310,6 @@ show_help(bool
   }
   fprintf(handle,
 "\n"
-"If no option -f is given, or if its value is `dflt',\n"
-"no fallback script is used.\n"
-"\n");
-  fprintf(handle,
 #ifdef BUILD_GUI
 "A command-line version of this program is called `ttfautohint'.\n"
 #else
@@ -337,7 +335,7 @@ show_version()
 #else
 "ttfautohint " VERSION "\n"
 #endif
-"Copyright (C) 2011-2013 Werner Lemberg <wl@gnu.org>.\n"
+"Copyright (C) 2011-2014 Werner Lemberg <wl@gnu.org>.\n"
 "License: FreeType License (FTL) or GNU GPLv2.\n"
 "This is free software: you are free to change and redistribute it.\n"
 "There is NO WARRANTY, to the extent permitted by law.\n");
@@ -371,6 +369,8 @@ main(int argc,
   bool no_info = false;
   bool symbol = false;
 
+  const char* default_script = NULL;
+  bool have_default_script = false;
   const char* fallback_script = NULL;
   bool have_fallback_script = false;
   const char* x_height_snapping_exceptions_string = NULL;
@@ -415,6 +415,7 @@ main(int argc,
 #ifndef BUILD_GUI
       {"debug", no_argument, NULL, DEBUG_OPTION},
 #endif
+      {"default-script", required_argument, NULL, 'D'},
       {"dehint", no_argument, NULL, 'd'},
       {"fallback-script", required_argument, NULL, 'f'},
       {"hinting-limit", required_argument, NULL, 'G'},
@@ -461,7 +462,7 @@ main(int argc,
     };
 
     int option_index;
-    int c = getopt_long_only(argc, argv, "cdf:G:hil:npr:stVvw:Wx:X:",
+    int c = getopt_long_only(argc, argv, "cdD:f:G:hil:npr:stVvw:Wx:X:",
                              long_options, &option_index);
     if (c == -1)
       break;
@@ -476,6 +477,11 @@ main(int argc,
       dehint = true;
       break;
 
+    case 'D':
+      default_script = optarg;
+      have_default_script = true;
+      break;
+
     case 'f':
       fallback_script = optarg;
       have_fallback_script = true;
@@ -585,6 +591,7 @@ main(int argc,
   if (dehint)
   {
     // -d makes ttfautohint ignore all other hinting options
+    have_default_script = false;
     have_fallback_script = false;
     have_hinting_range_min = false;
     have_hinting_range_max = false;
@@ -593,8 +600,10 @@ main(int argc,
     have_x_height_snapping_exceptions_string = false;
   }
 
+  if (!have_default_script)
+    default_script = "latn";
   if (!have_fallback_script)
-    fallback_script = "dflt";
+    fallback_script = "none";
   if (!have_hinting_range_min)
     hinting_range_min = TA_HINTING_RANGE_MIN;
   if (!have_hinting_range_max)
@@ -667,12 +676,27 @@ main(int argc,
                         "  \"%s\"\n"
                         "   %*s\n",
                         x_height_snapping_exceptions_string,
-                        s - x_height_snapping_exceptions_string + 1, "^");
+                        int(s - x_height_snapping_exceptions_string + 1),
+                        "^");
       }
       exit(EXIT_FAILURE);
     }
   }
 
+  if (have_default_script)
+  {
+    const Script_Names* sn;
+
+    for (sn = script_names; sn->tag; sn++)
+      if (!strcmp(default_script, sn->tag))
+        break;
+    if (!sn->tag)
+    {
+      fprintf(stderr, "Unknown script tag `%s'\n", default_script);
+      exit(EXIT_FAILURE);
+    }
+  }
+
   if (have_fallback_script)
   {
     const Script_Names* sn;
@@ -766,6 +790,9 @@ main(int argc,
     info_data.x_height_snapping_exceptions = x_height_snapping_exceptions;
     info_data.symbol = symbol;
 
+    strncpy(info_data.default_script,
+            default_script,
+            sizeof (info_data.default_script));
     strncpy(info_data.fallback_script,
             fallback_script,
             sizeof (info_data.fallback_script));
@@ -797,7 +824,7 @@ main(int argc,
                  "ignore-restrictions, windows-compatibility,"
                  "pre-hinting, hint-composites,"
                  "increase-x-height, x-height-snapping-exceptions,"
-                 "fallback-script, symbol,"
+                 "default-script, fallback-script, symbol,"
                  "dehint, debug",
                  in, out,
                  hinting_range_min, hinting_range_max, hinting_limit,
@@ -809,7 +836,7 @@ main(int argc,
                  ignore_restrictions, windows_compatibility,
                  pre_hinting, hint_composites,
                  increase_x_height, x_height_snapping_exceptions_string,
-                 fallback_script, symbol,
+                 default_script, fallback_script, symbol,
                  dehint, debug);
 
   if (!no_info)
@@ -849,9 +876,11 @@ main(int argc,
               "No symbol character map.\n");
     else if (error == TA_Err_Missing_Glyph)
       fprintf(stderr,
-              "No glyph for the key character"
+              "No glyph for a standard character"
               " to derive standard width and height.\n"
-              "For the latin script, this key character is `o' (U+006F).\n");
+              "Please check the documentation for a list of"
+              " script-specific standard characters,\n"
+              "or use option `--symbol'.\n");
     else
       fprintf(stderr,
               "Error code `0x%02x' while autohinting font:\n"
@@ -883,30 +912,56 @@ main(int argc,
   app.setOrganizationName("FreeType");
   app.setOrganizationDomain("freetype.org");
 
-  Main_GUI gui(hinting_range_min, hinting_range_max, hinting_limit,
+  bool alternative_layout = false;
+  {
+    // Display the window off the screen -- to get proper window dimensions
+    // including the frame, the window manager must have a chance to
+    // decorate it.
+    //
+    // We don't want to change the default window positioning algorithm of
+    // the platform's window manager, so we create the main GUI window
+    // twice.
+    //
+    // The original idea, however, was to simply move the off-screen window
+    // back to the screen with
+    //
+    //   gui.move(100, 100);
+    //   gui.setAttribute(Qt::WA_Moved, false);
+    //   gui.show();
+    //
+    // (unsetting the `WA_Moved' attribute makes the window manager handle
+    // the previous call to `move' as a position suggestion instead of a
+    // request).  Unfortuntely, there seems to be a bug in Qt 4.8.4 which
+    // prevents any effect of unsetting `WA_Moved' if `show' has already
+    // been called.
+
+    Main_GUI dummy(alternative_layout,
+                   hinting_range_min, hinting_range_max, hinting_limit,
+                   gray_strong_stem_width, gdi_cleartype_strong_stem_width,
+                   dw_cleartype_strong_stem_width, increase_x_height,
+                   x_height_snapping_exceptions_string,
+                   ignore_restrictions, windows_compatibility, pre_hinting,
+                   hint_composites, no_info, default_script, fallback_script,
+                   symbol, dehint);
+
+    dummy.move(-50000, -50000);
+    dummy.show();
+
+    // if the vertical size of our window is too large,
+    // select a horizontal layout
+    QRect screen(QApplication::desktop()->availableGeometry());
+    if (dummy.frameGeometry().height() > screen.width())
+      alternative_layout = true;
+  }
+
+  Main_GUI gui(alternative_layout,
+               hinting_range_min, hinting_range_max, hinting_limit,
                gray_strong_stem_width, gdi_cleartype_strong_stem_width,
                dw_cleartype_strong_stem_width, increase_x_height,
                x_height_snapping_exceptions_string,
                ignore_restrictions, windows_compatibility, pre_hinting,
-               hint_composites, no_info, fallback_script, symbol,
-               dehint);
-
-  // display the window off the screen --
-  // to get proper window dimensions including the frame,
-  // the window manager must have a chance to decorate it
-  gui.move(-50000, -50000);
-  gui.show();
-
-  // if the vertical size of our window is too large,
-  // select a horizontal layout
-  QRect screen(QApplication::desktop()->availableGeometry());
-  if (gui.frameGeometry().height() > screen.width())
-  {
-    gui.create_alternative_layout();
-    gui.adjustSize();
-  }
-
-  gui.move(0, 0);
+               hint_composites, no_info, default_script, fallback_script,
+               symbol, dehint);
   gui.show();
 
   return app.exec();
