@@ -1,6 +1,6 @@
 // maingui.cpp
 
-// Copyright (C) 2012-2013 by Werner Lemberg.
+// Copyright (C) 2012-2014 by Werner Lemberg.
 //
 // This file is part of the ttfautohint library, and may only be used,
 // modified, and distributed under the terms given in `COPYING'.  By
@@ -46,7 +46,7 @@ typedef struct Script_Names_
 } Script_Names;
 
 #undef SCRIPT
-#define SCRIPT(s, S, d) \
+#define SCRIPT(s, S, d, h, sc1, sc2, sc3) \
           {#s, d},
 
 const Script_Names script_names[] =
@@ -56,7 +56,8 @@ const Script_Names script_names[] =
 };
 
 
-Main_GUI::Main_GUI(int range_min,
+Main_GUI::Main_GUI(bool horizontal_layout,
+                   int range_min,
                    int range_max,
                    int limit,
                    bool gray,
@@ -69,6 +70,7 @@ Main_GUI::Main_GUI(int range_min,
                    bool pre,
                    bool composites,
                    bool no,
+                   const char* dflt,
                    const char* fallback,
                    bool symb,
                    bool dh)
@@ -89,22 +91,34 @@ Main_GUI::Main_GUI(int range_min,
   dehint(dh)
 {
   int i;
-  int dflt_script_idx = 0;
+
+  // map default script tag to an index,
+  // replacing an invalid one with the default value
+  int latn_script_idx = 0;
+  for (i = 0; script_names[i].tag; i++)
+  {
+    if (!strcmp("latn", script_names[i].tag))
+      latn_script_idx = i;
+    if (!strcmp(dflt, script_names[i].tag))
+      break;
+  }
+  default_script_idx = script_names[i].tag ? i : latn_script_idx;
 
   // map fallback script tag to an index,
   // replacing an invalid one with the default value
+  int none_script_idx = 0;
   for (i = 0; script_names[i].tag; i++)
   {
-    if (!strcmp("dflt", script_names[i].tag))
-      dflt_script_idx = i;
+    if (!strcmp("none", script_names[i].tag))
+      none_script_idx = i;
     if (!strcmp(fallback, script_names[i].tag))
       break;
   }
-  fallback_script_idx = script_names[i].tag ? i : dflt_script_idx;
+  fallback_script_idx = script_names[i].tag ? i : none_script_idx;
 
   x_height_snapping_exceptions = NULL;
 
-  create_layout();
+  create_layout(horizontal_layout);
   create_connections();
   create_actions();
   create_menus();
@@ -145,7 +159,7 @@ Main_GUI::about()
   QMessageBox::about(this,
                      tr("About TTFautohint"),
                      tr("<p>This is <b>TTFautohint</b> version %1<br>"
-                        " Copyright %2 2011-2013<br>"
+                        " Copyright %2 2011-2014<br>"
                         " by Werner Lemberg <tt>&lt;wl@gnu.org&gt;</tt></p>"
                         ""
                         "<p><b>TTFautohint</b> adds new auto-generated hints"
@@ -240,6 +254,8 @@ Main_GUI::check_dehint()
     max_label->setEnabled(false);
     max_box->setEnabled(false);
 
+    default_label->setEnabled(false);
+    default_box->setEnabled(false);
     fallback_label->setEnabled(false);
     fallback_box->setEnabled(false);
 
@@ -272,6 +288,8 @@ Main_GUI::check_dehint()
     max_label->setEnabled(true);
     max_box->setEnabled(true);
 
+    default_label->setEnabled(true);
+    default_box->setEnabled(true);
     fallback_label->setEnabled(true);
     fallback_box->setEnabled(true);
 
@@ -692,12 +710,12 @@ Main_GUI::handle_error(TA_Error error,
     QMessageBox::warning(
       this,
       "TTFautohint",
-      tr("No glyph for the key character"
-         " to derive standard stem width and height.\n"
-         "For the latin script, this key character is %1 (U+006F).\n"
+      tr("No glyph for a standard character"
+         " to derive standard width and height.\n"
+         "Please check the documentation for a list of"
+         " script-specific standard characters.\n"
          "\n"
-         "Set the %2 checkbox if you want to circumvent this test.")
-         .arg(QUOTE_STRING_LITERAL("o"))
+         "Set the %1 checkbox if you want to circumvent this test.")
          .arg(QUOTE_STRING_LITERAL("symbol")),
       QMessageBox::Ok,
       QMessageBox::Ok);
@@ -785,6 +803,9 @@ again:
   info_data.symbol = symbol_box->isChecked();
   info_data.dehint = dehint_box->isChecked();
 
+  strncpy(info_data.default_script,
+          script_names[default_box->currentIndex()].tag,
+          sizeof (info_data.default_script));
   strncpy(info_data.fallback_script,
           script_names[fallback_box->currentIndex()].tag,
           sizeof (info_data.fallback_script));
@@ -830,8 +851,8 @@ again:
                  "hint-composites,"
                  "increase-x-height,"
                  "x-height-snapping-exceptions,"
-                 "fallback-script, symbol,"
-                 "dehint",
+                 "default-script, fallback-script,"
+                 "symbol, dehint",
                  input, output,
                  info_data.hinting_range_min, info_data.hinting_range_max,
                  info_data.hinting_limit,
@@ -847,8 +868,8 @@ again:
                  info_data.hint_composites,
                  info_data.increase_x_height,
                  snapping_string.constData(),
-                 info_data.fallback_script, info_data.symbol,
-                 info_data.dehint);
+                 info_data.default_script, info_data.fallback_script,
+                 info_data.symbol, info_data.dehint);
 
   if (info_box->isChecked())
   {
@@ -872,7 +893,7 @@ again:
 // XXX distances are specified in pixels,
 //     making the layout dependent on the output device resolution
 void
-Main_GUI::create_layout()
+Main_GUI::create_layout(bool horizontal_layout)
 {
   //
   // file stuff
@@ -901,19 +922,6 @@ Main_GUI::create_layout()
        " to the input font but will contain new, generated hints."));
   output_line->setCompleter(completer);
 
-  // layout
-  QGridLayout* file_layout = new QGridLayout;
-
-  file_layout->addWidget(input_label, 0, 0, Qt::AlignRight);
-  file_layout->addWidget(input_line, 0, 1);
-  file_layout->addWidget(input_button, 0, 2);
-
-  file_layout->setRowStretch(1, 1);
-
-  file_layout->addWidget(output_label, 2, 0, Qt::AlignRight);
-  file_layout->addWidget(output_line, 2, 1);
-  file_layout->addWidget(output_button, 2, 2);
-
   //
   // minmax controls
   //
@@ -947,6 +955,26 @@ Main_GUI::create_layout()
   max_box->setKeyboardTracking(false);
   max_box->setRange(2, 10000);
 
+  //
+  // OpenType default script
+  //
+  default_label = new QLabel(tr("Defa&ult Script:"));
+  default_box = new QComboBox;
+  default_label->setBuddy(default_box);
+  default_label->setToolTip(
+    tr("This sets the default script for OpenType features:"
+       "  After applying all features that are handled specially"
+       " (for example small caps or superscript glyphs),"
+       " <b>TTFautohint</b> uses this value for the remaining features."));
+  for (int i = 0; script_names[i].tag; i++)
+  {
+    // XXX: how to provide translations?
+    default_box->insertItem(i,
+                            QString("%1 (%2)")
+                                    .arg(script_names[i].tag)
+                                    .arg(script_names[i].description));
+  }
+
   //
   // hinting and fallback controls
   //
@@ -954,7 +982,7 @@ Main_GUI::create_layout()
   fallback_box = new QComboBox;
   fallback_label->setBuddy(fallback_box);
   fallback_label->setToolTip(
-    tr("This sets the fallback script module for glyphs"
+    tr("This sets the fallback script for glyphs"
        " that <b>TTFautohint</b> can't map to a script automatically."));
   for (int i = 0; script_names[i].tag; i++)
   {
@@ -1038,8 +1066,9 @@ Main_GUI::create_layout()
   pre_box = new QCheckBox(tr("Pr&e-hinting"), this);
   pre_box->setToolTip(
     tr("If switched on, the original bytecode of the input font"
-       " gets applied before <b>TTFautohint</b> starts processing"
-       " the outlines of the glyphs."));
+       " gets applied (at EM size, usually 2048ppem)"
+       " to derive the glyph outlines for <b>TTFautohint</b>."
+       "  Note that the original bytecode will always be discarded."));
 
   hint_box = new QCheckBox(tr("Hint Co&mposites")
                            + "                ", this); // make label wider
@@ -1116,6 +1145,31 @@ Main_GUI::create_layout()
                                + tr("&Run")
                                + "    "); // make label wider
 
+  if (horizontal_layout)
+    create_horizontal_layout();
+  else
+    create_vertical_layout();
+}
+
+
+// XXX distances are specified in pixels,
+//     making the layout dependent on the output device resolution
+void
+Main_GUI::create_vertical_layout()
+{
+  // top area
+  QGridLayout* file_layout = new QGridLayout;
+
+  file_layout->addWidget(input_label, 0, 0, Qt::AlignRight);
+  file_layout->addWidget(input_line, 0, 1);
+  file_layout->addWidget(input_button, 0, 2);
+
+  file_layout->setRowStretch(1, 1);
+
+  file_layout->addWidget(output_label, 2, 0, Qt::AlignRight);
+  file_layout->addWidget(output_line, 2, 1);
+  file_layout->addWidget(output_button, 2, 2);
+
   //
   // the whole gui
   //
@@ -1144,6 +1198,8 @@ Main_GUI::create_layout()
   gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
   gui_layout->setRowStretch(row++, 1);
 
+  gui_layout->addWidget(default_label, row, 0, Qt::AlignRight);
+  gui_layout->addWidget(default_box, row++, 1, Qt::AlignLeft);
   gui_layout->addWidget(fallback_label, row, 0, Qt::AlignRight);
   gui_layout->addWidget(fallback_box, row++, 1, Qt::AlignLeft);
 
@@ -1195,85 +1251,87 @@ Main_GUI::create_layout()
 // XXX distances are specified in pixels,
 //     making the layout dependent on the output device resolution
 void
-Main_GUI::create_alternative_layout()
+Main_GUI::create_horizontal_layout()
 {
   // top area
-  QGridLayout* file_alt_layout = new QGridLayout;
+  QGridLayout* file_layout = new QGridLayout;
 
-  file_alt_layout->addWidget(input_label, 0, 0, Qt::AlignRight);
-  file_alt_layout->addWidget(input_line, 0, 1);
-  file_alt_layout->addWidget(input_button, 0, 2);
+  file_layout->addWidget(input_label, 0, 0, Qt::AlignRight);
+  file_layout->addWidget(input_line, 0, 1);
+  file_layout->addWidget(input_button, 0, 2);
 
-  file_alt_layout->setRowStretch(1, 1);
+  file_layout->setRowStretch(1, 1);
 
-  file_alt_layout->addWidget(output_label, 2, 0, Qt::AlignRight);
-  file_alt_layout->addWidget(output_line, 2, 1);
-  file_alt_layout->addWidget(output_button, 2, 2);
+  file_layout->addWidget(output_label, 2, 0, Qt::AlignRight);
+  file_layout->addWidget(output_line, 2, 1);
+  file_layout->addWidget(output_button, 2, 2);
 
-  QGridLayout* gui_alt_layout = new QGridLayout;
+  QGridLayout* gui_layout = new QGridLayout;
   QFrame* hline = new QFrame;
   hline->setFrameShape(QFrame::HLine);
   int row = 0; // this counter simplifies inserting new items
 
   // margin
-  gui_alt_layout->setColumnMinimumWidth(0, 10); // XXX urgh, pixels...
-  gui_alt_layout->setColumnStretch(0, 1);
+  gui_layout->setColumnMinimumWidth(0, 10); // XXX urgh, pixels...
+  gui_layout->setColumnStretch(0, 1);
 
   // left
-  gui_alt_layout->setRowMinimumHeight(row, 10); // XXX urgh, pixels...
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->setRowMinimumHeight(row, 10); // XXX urgh, pixels...
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addLayout(file_alt_layout, row, 0, row, -1);
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->addLayout(file_layout, row, 0, row, -1);
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addWidget(hline, row, 0, row, -1);
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->addWidget(hline, row, 0, row, -1);
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addWidget(min_label, row, 1, Qt::AlignRight);
-  gui_alt_layout->addWidget(min_box, row++, 2, Qt::AlignLeft);
-  gui_alt_layout->addWidget(max_label, row, 1, Qt::AlignRight);
-  gui_alt_layout->addWidget(max_box, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(min_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(min_box, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(max_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(max_box, row++, 2, Qt::AlignLeft);
 
-  gui_alt_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addWidget(fallback_label, row, 1, Qt::AlignRight);
-  gui_alt_layout->addWidget(fallback_box, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(default_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(default_box, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(fallback_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(fallback_box, row++, 2, Qt::AlignLeft);
 
-  gui_alt_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addWidget(limit_label, row, 1, Qt::AlignRight);
-  gui_alt_layout->addWidget(limit_box, row++, 2, Qt::AlignLeft);
-  gui_alt_layout->addWidget(no_limit_box, row++, 2);
+  gui_layout->addWidget(limit_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(limit_box, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(no_limit_box, row++, 2);
 
-  gui_alt_layout->addWidget(increase_label, row, 1, Qt::AlignRight);
-  gui_alt_layout->addWidget(increase_box, row++, 2, Qt::AlignLeft);
-  gui_alt_layout->addWidget(no_increase_box, row++, 2);
+  gui_layout->addWidget(increase_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(increase_box, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(no_increase_box, row++, 2);
 
-  gui_alt_layout->addWidget(snapping_label, row, 1, Qt::AlignRight);
-  gui_alt_layout->addWidget(snapping_line, row++, 2, Qt::AlignLeft);
+  gui_layout->addWidget(snapping_label, row, 1, Qt::AlignRight);
+  gui_layout->addWidget(snapping_line, row++, 2, Qt::AlignLeft);
 
   // column separator
-  gui_alt_layout->setColumnMinimumWidth(3, 20); // XXX urgh, pixels...
-  gui_alt_layout->setColumnStretch(3, 1);
+  gui_layout->setColumnMinimumWidth(3, 20); // XXX urgh, pixels...
+  gui_layout->setColumnStretch(3, 1);
 
   // right
   row = 4;
-  gui_alt_layout->addWidget(wincomp_box, row++, 4);
-  gui_alt_layout->addWidget(pre_box, row++, 4);
-  gui_alt_layout->addWidget(hint_box, row++, 4);
-  gui_alt_layout->addWidget(symbol_box, row++, 4);
-  gui_alt_layout->addWidget(dehint_box, row++, 4);
-  gui_alt_layout->addWidget(info_box, row++, 4);
+  gui_layout->addWidget(wincomp_box, row++, 4);
+  gui_layout->addWidget(pre_box, row++, 4);
+  gui_layout->addWidget(hint_box, row++, 4);
+  gui_layout->addWidget(symbol_box, row++, 4);
+  gui_layout->addWidget(dehint_box, row++, 4);
+  gui_layout->addWidget(info_box, row++, 4);
 
-  gui_alt_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addWidget(stem_label, row++, 4);
+  gui_layout->addWidget(stem_label, row++, 4);
 
   QGridLayout* stem_layout = new QGridLayout;
   stem_layout->setColumnMinimumWidth(0, 20); // XXX urgh, pixels...
@@ -1281,23 +1339,22 @@ Main_GUI::create_alternative_layout()
   stem_layout->addWidget(gdi_box, 1, 1);
   stem_layout->addWidget(dw_box, 2, 1);
 
-  gui_alt_layout->addLayout(stem_layout, row, 4, 3, 1);
+  gui_layout->addLayout(stem_layout, row, 4, 3, 1);
   row += 3;
 
-  gui_alt_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
-  gui_alt_layout->setRowStretch(row++, 1);
+  gui_layout->setRowMinimumHeight(row, 20); // XXX urgh, pixels...
+  gui_layout->setRowStretch(row++, 1);
 
-  gui_alt_layout->addWidget(run_button, row++, 4, Qt::AlignRight);
+  gui_layout->addWidget(run_button, row++, 4, Qt::AlignRight);
 
   // margin
-  gui_alt_layout->setColumnMinimumWidth(5, 10); // XXX urgh, pixels...
-  gui_alt_layout->setColumnStretch(5, 1);
+  gui_layout->setColumnMinimumWidth(5, 10); // XXX urgh, pixels...
+  gui_layout->setColumnStretch(5, 1);
 
   // create dummy widget to register layout
-  QWidget* main_alt_widget = new QWidget;
-  main_alt_widget->setLayout(gui_alt_layout);
-  // setCentralWidget automatically deletes the old widget
-  setCentralWidget(main_alt_widget);
+  QWidget* main_widget = new QWidget;
+  main_widget->setLayout(gui_layout);
+  setCentralWidget(main_widget);
   setWindowTitle("TTFautohint");
 }
 
@@ -1386,6 +1443,7 @@ Main_GUI::set_defaults()
   min_box->setValue(hinting_range_min);
   max_box->setValue(hinting_range_max);
 
+  default_box->setCurrentIndex(default_script_idx);
   fallback_box->setCurrentIndex(fallback_script_idx);
 
   limit_box->setValue(hinting_limit ? hinting_limit : hinting_range_max);
