@@ -1,7 +1,7 @@
 /* tacvt.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -19,11 +19,12 @@
 static FT_Error
 TA_sfnt_compute_global_hints(SFNT* sfnt,
                              FONT* font,
-                             FT_UInt script_idx)
+                             TA_Style style_idx)
 {
   FT_Error error;
   FT_Face face = sfnt->face;
-  FT_UInt idx;
+  FT_ULong glyph_index;
+  FT_Long y_offset;
   FT_Int32 load_flags;
 
 
@@ -40,20 +41,81 @@ TA_sfnt_compute_global_hints(SFNT* sfnt,
       return TA_Err_Missing_Unicode_CMap;
   }
 
-  if (font->symbol)
-    idx = 0;
-  else
   {
-    /* load standard character to trigger script initializations */
-    /* XXX make this configurable to use a different letter */
-    idx = FT_Get_Char_Index(face,
-                            ta_script_classes[script_idx]->standard_char);
-    if (!idx)
+    TA_FaceGlobals globals = (TA_FaceGlobals)sfnt->face->autohint.data;
+    FT_Byte* gstyles = globals->glyph_styles;
+
+    TA_StyleClass style_class = ta_style_classes[style_idx];
+    TA_ScriptClass script_class = ta_script_classes[style_class->script];
+
+    TA_StyleMetricsRec dummy;
+
+
+    /* we don't have a `TA_Loader' object yet */
+    dummy.globals = globals;
+    dummy.style_class = style_class;
+
+    /*
+     * We check more than a single standard character to catch features
+     * like `c2sc' (small caps from caps) that don't contain lowercase
+     * letters by definition, or other features that mainly operate on
+     * numerals.
+     */
+    ta_get_char_index(&dummy,
+                      script_class->standard_char1,
+                      &glyph_index,
+                      &y_offset);
+    if (!glyph_index)
+    {
+      if (script_class->standard_char2)
+      {
+        ta_get_char_index(&dummy,
+                          script_class->standard_char2,
+                          &glyph_index,
+                          &y_offset);
+        if (!glyph_index)
+        {
+          if (script_class->standard_char3)
+            ta_get_char_index(&dummy,
+                              script_class->standard_char3,
+                              &glyph_index,
+                              &y_offset);
+        }
+      }
+    }
+
+    if (!glyph_index)
       return TA_Err_Missing_Glyph;
+
+    /*
+     * We now know that HarfBuzz can access the standard character in the
+     * current OpenType feature.  However, this doesn't guarantee that there
+     * actually *is* a standard character in the corresponding coverage,
+     * since glyphs shifted with data from the GPOS table are ignored in the
+     * coverage (but neverless used to derive stem widths).  For this
+     * reason, search an arbitrary character from the current coverage to
+     * trigger the coverage's metrics computation.
+     */
+    if (gstyles[glyph_index] != style_idx)
+    {
+      FT_Int i;
+
+
+      for (i = 0; i < globals->glyph_count; i++)
+      {
+        if (gstyles[i] == style_idx)
+          break;
+      }
+
+      if (i == globals->glyph_count)
+        return TA_Err_Missing_Glyph;
+
+      glyph_index = i;
+    }
   }
 
   load_flags = 1 << 29; /* vertical hinting only */
-  error = ta_loader_load_glyph(font, face, idx, load_flags);
+  error = ta_loader_load_glyph(font, face, glyph_index, load_flags);
 
   return error;
 }
@@ -75,7 +137,7 @@ TA_table_build_cvt(FT_Byte** cvt,
   FT_UInt vwidth_count;
   FT_UInt blue_count;
 
-  FT_UInt i, j, i_max;
+  FT_UInt i, j;
   FT_UInt buf_len;
   FT_UInt len;
   FT_Byte* buf;
@@ -85,37 +147,34 @@ TA_table_build_cvt(FT_Byte** cvt,
   FT_Error error;
 
 
-  /* checking multiple scripts doesn't make sense for symbol fonts */
-  i_max = font->symbol ? 1 : TA_SCRIPT_MAX;
-
-  /* loop over all scripts and collect the relevant CVT data */
+  /* loop over all styles and collect the relevant CVT data */
   /* to compute the necessary array sizes and meta-information */
   hwidth_count = 0;
   vwidth_count = 0;
   blue_count = 0;
 
-  data->num_used_scripts = 0;
+  data->num_used_styles = 0;
 
-  for (i = 0; i < i_max; i++)
+  for (i = 0; i < TA_STYLE_MAX; i++)
   {
-    error = TA_sfnt_compute_global_hints(sfnt, font, i);
+    error = TA_sfnt_compute_global_hints(sfnt, font, (TA_Style)i);
     if (error == TA_Err_Missing_Glyph)
     {
       TA_FaceGlobals globals = (TA_FaceGlobals)sfnt->face->autohint.data;
-      FT_Byte* gscripts = globals->glyph_scripts;
+      FT_Byte* gstyles = globals->glyph_styles;
       FT_Int nn;
 
 
-      data->script_ids[i] = 0xFFFFU;
+      data->style_ids[i] = 0xFFFFU;
 
-      /* remove all references to this script; */
+      /* remove all references to this style; */
       /* otherwise blue zones are computed later on, which we don't want */
       for (nn = 0; nn < globals->glyph_count; nn++)
       {
-        if ((gscripts[nn] & ~TA_DIGIT) == i)
+        if ((gstyles[nn] & ~TA_DIGIT) == i)
         {
-          gscripts[nn] &= ~TA_SCRIPT_NONE;
-          gscripts[nn] |= globals->font->fallback_script;
+          gstyles[nn] &= ~TA_STYLE_UNASSIGNED;
+          gstyles[nn] |= globals->font->fallback_style;
         }
       }
 
@@ -124,9 +183,9 @@ TA_table_build_cvt(FT_Byte** cvt,
     if (error)
       return error;
 
-    data->script_ids[i] = data->num_used_scripts++;
+    data->style_ids[i] = data->num_used_styles++;
 
-    if (font->loader->hints.metrics->script_class->script == TA_SCRIPT_DFLT)
+    if (i == TA_STYLE_NONE_DFLT)
       continue;
     else
     {
@@ -142,14 +201,15 @@ TA_table_build_cvt(FT_Byte** cvt,
     }
   }
 
-  /* exit if the font doesn't contain a single supported script */
-  if (!data->num_used_scripts)
+  /* exit if the font doesn't contain a single supported style, */
+  /* and we don't have a symbol font */
+  if (!data->num_used_styles && !font->symbol)
     return TA_Err_Missing_Glyph;
 
   buf_len = cvtl_max_runtime /* runtime values 1 */
-            + data->num_used_scripts /* runtime values 2 (for scaling) */
-            + 2 * data->num_used_scripts /* runtime values 3 (blue data) */
-            + 2 * data->num_used_scripts /* vert. and horiz. std. widths */
+            + data->num_used_styles /* runtime values 2 (for scaling) */
+            + 2 * data->num_used_styles /* runtime values 3 (blue data) */
+            + 2 * data->num_used_styles /* vert. and horiz. std. widths */
             + hwidth_count
             + vwidth_count
             + 2 * blue_count; /* round and flat blue zones */
@@ -172,30 +232,30 @@ TA_table_build_cvt(FT_Byte** cvt,
    * some CVT values are initialized (and modified) at runtime:
    *
    *   (1) the `cvtl_xxx' values (see `tabytecode.h')
-   *   (2) a scaling value for each script
+   *   (2) a scaling value for each style
    *   (3) offset and size of the vertical widths array
-   *       (needed by `bci_{smooth,strong}_stem_width') for each script
+   *       (needed by `bci_{smooth,strong}_stem_width') for each style
    */
   for (i = 0; i < (cvtl_max_runtime
-                   + data->num_used_scripts
-                   + 2 * data->num_used_scripts) * 2; i++)
+                   + data->num_used_styles
+                   + 2 * data->num_used_styles) * 2; i++)
     *(bufp++) = 0;
 
   cvt_offset = bufp - buf;
 
-  /* loop again over all scripts and copy CVT data */
-  for (i = 0; i < i_max; i++)
+  /* loop again over all styles and copy CVT data */
+  for (i = 0; i < TA_STYLE_MAX; i++)
   {
     /* collect offsets */
     data->cvt_offsets[i] = ((FT_UInt)(bufp - buf) - cvt_offset) >> 1;
 
-    error = TA_sfnt_compute_global_hints(sfnt, font, i);
+    error = TA_sfnt_compute_global_hints(sfnt, font, (TA_Style)i);
     if (error == TA_Err_Missing_Glyph)
       continue;
     if (error)
       return error;
 
-    if (font->loader->hints.metrics->script_class->script == TA_SCRIPT_DFLT)
+    if (font->loader->hints.metrics->style_class == &ta_none_dflt_style_class)
     {
       haxis = NULL;
       vaxis = NULL;
