@@ -1,7 +1,7 @@
 /* tadummy.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -23,13 +23,10 @@
 #include "tatypes.h"
 
 
-/* a dummy writing system and script class used when no hinting should be */
-/* performed */
+/* a dummy writing system used when no hinting should be performed */
 
 extern const TA_WritingSystemClassRec ta_dummy_writing_system_class;
 
-extern const TA_ScriptClassRec ta_dflt_script_class;
-
 #endif /* __TADUMMY_H__ */
 
 /* end of tadummy.h */
