@@ -1,7 +1,7 @@
 /* tahints.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -22,11 +22,15 @@
 
 #include "tatypes.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #define xxTA_SORT_SEGMENTS
 
 
 /* the definition of outline glyph hints; these are shared */
-/* by all script analysis routines (until now) */
+/* by all writing system analysis routines (until now) */
 
 typedef enum TA_Dimension_
 {
@@ -230,15 +234,13 @@ typedef enum TA_Direction_
 /* all inflection points in the outline have this flag set */
 #define TA_FLAG_INFLECTION (1 << 9)
 
-/* the current point is very near to another one */
-#define TA_FLAG_NEAR (1 << 10)
-
 
 /* edge hint flags */
 #define TA_EDGE_NORMAL 0
 #define TA_EDGE_ROUND (1 << 0)
 #define TA_EDGE_SERIF (1 << 1)
 #define TA_EDGE_DONE (1 << 2)
+#define TA_EDGE_NEUTRAL (1 << 3)
 
 
 typedef struct TA_PointRec_* TA_Point;
@@ -421,8 +423,8 @@ typedef struct TA_GlyphHintsRec_
   TA_AxisHintsRec axis[TA_DIMENSION_MAX];
 
   FT_UInt32 scaler_flags; /* copy of scaler flags */
-  FT_UInt32 other_flags; /* free for script-specific implementations */
-  TA_ScriptMetrics metrics;
+  FT_UInt32 other_flags; /* free for style-specific implementations */
+  TA_StyleMetrics metrics;
 
   FT_Pos xmin_delta; /* used for warping */
   FT_Pos xmax_delta;
@@ -502,7 +504,7 @@ ta_glyph_hints_init(TA_GlyphHints hints);
 
 void
 ta_glyph_hints_rescale(TA_GlyphHints hints,
-                       TA_ScriptMetrics metrics);
+                       TA_StyleMetrics metrics);
 
 FT_Error
 ta_glyph_hints_reload(TA_GlyphHints hints,
@@ -543,6 +545,11 @@ ta_glyph_hints_done(TA_GlyphHints hints);
           (((seg1)->pos > (seg2)->pos) ? (seg1)->pos - (seg2)->pos \
                                        : (seg2)->pos - (seg1)->pos)
 
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* __TAHINTS_H__ */
 
+
 /* end of tahints.h */
