@@ -150,7 +150,13 @@ ta_latin_metrics_init_widths(TA_LatinMetrics metrics,
       if (error)
         goto Exit;
 
-      ta_latin_hints_link_segments(hints, (TA_Dimension)dim);
+      /*
+       * We assume that the glyphs selected for the stem width
+       * computation are `featureless' enough so that the linking
+       * algorithm works fine without adjustments of its scoring
+       * function.
+       */
+      ta_latin_hints_link_segments(hints, 0, NULL, (TA_Dimension)dim);
 
       seg = axhints->segments;
       limit = seg + axhints->num_segments;
@@ -277,6 +283,14 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
           have_flag = 1;
         }
 
+        if (TA_LATIN_IS_NEUTRAL_BLUE(bs))
+        {
+          if (have_flag)
+            TA_LOG_GLOBAL((", "));
+          TA_LOG_GLOBAL(("neutral"));
+          have_flag = 1;
+        }
+
         if (TA_LATIN_IS_X_HEIGHT_BLUE(bs))
         {
           if (have_flag)
@@ -667,6 +681,13 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
                           || FT_CURVE_TAG(outline.tags[best_segment_last])
                                != FT_CURVE_TAG_ON);
 
+        if (round && TA_LATIN_IS_NEUTRAL_BLUE(bs))
+        {
+          /* only use flat segments for a neutral blue zone */
+          TA_LOG_GLOBAL((" (round, skipped)\n"));
+          continue;
+        }
+
         TA_LOG_GLOBAL((" (%s)\n", round ? "round" : "flat"));
       }
 
@@ -735,6 +756,8 @@ ta_latin_metrics_init_blues(TA_LatinMetrics metrics,
     blue->flags = 0;
     if (TA_LATIN_IS_TOP_BLUE(bs))
       blue->flags |= TA_LATIN_BLUE_TOP;
+    if (TA_LATIN_IS_NEUTRAL_BLUE(bs))
+      blue->flags |= TA_LATIN_BLUE_NEUTRAL;
 
     /* the following flag is used later to adjust the y and x scales */
     /* in order to optimize the pixel grid alignment */
@@ -1383,10 +1406,13 @@ Exit:
 }
 
 
-/* link segments to form stems and serifs */
+/* link segments to form stems and serifs; if `width_count' and */
+/* `widths' are non-zero, use them to fine-tune the scoring function */
 
 void
 ta_latin_hints_link_segments(TA_GlyphHints hints,
+                             FT_UInt width_count,
+                             TA_WidthRec* widths,
                              TA_Dimension dim)
 {
   TA_AxisHints axis = &hints->axis[dim];
@@ -1394,16 +1420,28 @@ ta_latin_hints_link_segments(TA_GlyphHints hints,
   TA_Segment segments = axis->segments;
   TA_Segment segment_limit = segments + axis->num_segments;
 
-  FT_Pos len_threshold, len_score;
+  FT_Pos len_threshold, len_score, dist_score, max_width;
   TA_Segment seg1, seg2;
 
 
+  if (width_count)
+    max_width = widths[width_count - 1].org;
+  else
+    max_width = 0;
+
+  /* a heuristic value to set up a minimum value for overlapping */
   len_threshold = TA_LATIN_CONSTANT(hints->metrics, 8);
   if (len_threshold == 0)
     len_threshold = 1;
 
+  /* a heuristic value to weight lengths */
   len_score = TA_LATIN_CONSTANT(hints->metrics, 6000);
 
+  /* a heuristic value to weight distances (no call to */
+  /* TA_LATIN_CONSTANT needed, since we work on multiples */
+  /* of the stem width) */
+  dist_score = 3000;
+
   /* now compare each segment to the others */
   for (seg1 = segments; seg1 < segment_limit; seg1++)
   {
@@ -1425,10 +1463,9 @@ ta_latin_hints_link_segments(TA_GlyphHints hints,
           && pos2 > pos1)
       {
         /* compute distance between the two segments */
-        FT_Pos dist = pos2 - pos1;
         FT_Pos min = seg1->min_coord;
         FT_Pos max = seg1->max_coord;
-        FT_Pos len, score;
+        FT_Pos len;
 
 
         if (min < seg2->min_coord)
@@ -1437,15 +1474,49 @@ ta_latin_hints_link_segments(TA_GlyphHints hints,
           max = seg2->max_coord;
 
         /* compute maximum coordinate difference of the two segments */
+        /* (this is, how much they overlap) */
         len = max - min;
         if (len >= len_threshold)
         {
-          /* small coordinate differences cause a higher score, and */
-          /* segments with a greater distance cause a higher score also */
-          score = dist + len_score / len;
+          /*
+           * The score is the sum of two demerits indicating the
+           * `badness' of a fit, measured along the segments' main axis
+           * and orthogonal to it, respectively.
+           *
+           * o The less overlapping along the main axis, the worse it
+           *   is, causing a larger demerit.
+           *
+           * o The nearer the orthogonal distance to a stem width, the
+           *   better it is, causing a smaller demerit.  For simplicity,
+           *   however, we only increase the demerit for values that
+           *   exceed the largest stem width.
+           */
+
+          FT_Pos dist = pos2 - pos1;
+
+          FT_Pos dist_demerit, score;
+
+
+          if (max_width)
+          {
+            /* distance demerits are based on multiples of `max_width'; */
+            /* we scale by 1024 for getting more precision */
+            FT_Pos delta = (dist << 10) / max_width - (1 << 10);
+
+
+            if (delta > 10000)
+              dist_demerit = 32000;
+            else if (delta > 0)
+              dist_demerit = delta * delta / dist_score;
+            else
+              dist_demerit = 0;
+          }
+          else
+            dist_demerit = dist; /* default if no widths available */
+
+          score = dist_demerit + len_score / len;
 
           /* and we search for the smallest score */
-          /* of the sum of the two values */
           if (score < seg1->score)
           {
             seg1->score = score;
@@ -1768,6 +1839,8 @@ Exit:
 
 FT_Error
 ta_latin_hints_detect_features(TA_GlyphHints hints,
+                               FT_UInt width_count,
+                               TA_WidthRec* widths,
                                TA_Dimension dim)
 {
   FT_Error error;
@@ -1776,7 +1849,7 @@ ta_latin_hints_detect_features(TA_GlyphHints hints,
   error = ta_latin_hints_compute_segments(hints, dim);
   if (!error)
   {
-    ta_latin_hints_link_segments(hints, dim);
+    ta_latin_hints_link_segments(hints, width_count, widths, dim);
 
     error = ta_latin_hints_compute_edges(hints, dim);
   }
@@ -1808,6 +1881,7 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
   {
     FT_UInt bb;
     TA_Width best_blue = NULL;
+    FT_Bool best_blue_is_neutral = 0;
     FT_Pos best_dist; /* initial threshold */
 
     FT_UInt best_blue_idx = 0;
@@ -1831,22 +1905,22 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
          bb++)
     {
       TA_LatinBlue blue = latin->blues + bb;
-      FT_Bool is_top_blue, is_major_dir;
+      FT_Bool is_top_blue, is_neutral_blue, is_major_dir;
 
 
       /* skip inactive blue zones (i.e., those that are too large) */
       if (!(blue->flags & TA_LATIN_BLUE_ACTIVE))
         continue;
 
-      /* if it is a top zone, check for right edges -- */
-      /* if it is a bottom zone, check for left edges */
+      /* if it is a top zone, check for right edges (against the major */
+      /* direction); if it is a bottom zone, check for left edges (in */
+      /* the major direction) */
       is_top_blue = (FT_Byte)((blue->flags & TA_LATIN_BLUE_TOP) != 0);
+      is_neutral_blue = (FT_Byte)((blue->flags & TA_LATIN_BLUE_NEUTRAL) != 0);
       is_major_dir = FT_BOOL(edge->dir == axis->major_dir);
 
-      /* if it is a top zone, the edge must be against the major */
-      /* direction; if it is a bottom zone, it must be in the major */
-      /* direction */
-      if (is_top_blue ^ is_major_dir)
+      /* neutral blue zones are handled for both directions */
+      if (is_top_blue ^ is_major_dir || is_neutral_blue)
       {
         FT_Pos dist;
 
@@ -1861,6 +1935,7 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
         {
           best_dist = dist;
           best_blue = &blue->ref;
+          best_blue_is_neutral = is_neutral_blue;
 
           best_blue_idx = bb;
           best_blue_is_shoot = 0;
@@ -1869,9 +1944,11 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
         /* now compare it to the overshoot position and check whether */
         /* the edge is rounded, and whether the edge is over the */
         /* reference position of a top zone, or under the reference */
-        /* position of a bottom zone */
+        /* position of a bottom zone (provided we don't have a */
+        /* neutral blue zone) */
         if (edge->flags & TA_EDGE_ROUND
-            && dist != 0)
+            && dist != 0
+            && !is_neutral_blue)
         {
           FT_Bool is_under_ref = FT_BOOL(edge->fpos < blue->ref.org);
 
@@ -1887,6 +1964,7 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
             {
               best_dist = dist;
               best_blue = &blue->shoot;
+              best_blue_is_neutral = is_neutral_blue;
 
               best_blue_idx = bb;
               best_blue_is_shoot = 1;
@@ -1901,6 +1979,8 @@ ta_latin_hints_compute_blue_edges(TA_GlyphHints hints,
       edge->blue_edge = best_blue;
       edge->best_blue_idx = best_blue_idx;
       edge->best_blue_is_shoot = best_blue_is_shoot;
+      if (best_blue_is_neutral)
+        edge->flags |= TA_EDGE_NEUTRAL;
     }
   }
 }
@@ -2022,7 +2102,7 @@ ta_latin_snap_width(TA_Width widths,
 /* compute the snapped width of a given stem, ignoring very thin ones */
 
 /* there is a lot of voodoo in this function; changing the hard-coded */
-/* parameters influence the whole hinting process */
+/* parameters influences the whole hinting process */
 
 static FT_Pos
 ta_latin_compute_stem_width(TA_GlyphHints hints,
@@ -2265,14 +2345,40 @@ ta_latin_hint_edges(TA_GlyphHints hints,
       if (edge->flags & TA_EDGE_DONE)
         continue;
 
-      blue = edge->blue_edge;
       edge1 = NULL;
       edge2 = edge->link;
 
+      /*
+       * If a stem contains both a neutral and a non-neutral blue zone,
+       * skip the neutral one.  Otherwise, outlines with different
+       * directions might be incorrectly aligned at the same vertical
+       * position.
+       *
+       * If we have two neutral blue zones, skip one of them.
+       */
+      if (edge->blue_edge && edge2 && edge2->blue_edge)
+      {
+        FT_Byte neutral = edge->flags & TA_EDGE_NEUTRAL;
+        FT_Byte neutral2 = edge2->flags & TA_EDGE_NEUTRAL;
+
+
+        if ((neutral && neutral2) || neutral2)
+        {
+          edge2->blue_edge = NULL;
+          edge2->flags &= ~TA_EDGE_NEUTRAL;
+        }
+        else if (neutral)
+        {
+          edge->blue_edge = NULL;
+          edge->flags &= ~TA_EDGE_NEUTRAL;
+        }
+      }
+
+      blue = edge->blue_edge;
       if (blue)
         edge1 = edge;
 
-      /* flip edges if the other stem is aligned to a blue zone */
+      /* flip edges if the other edge is aligned to a blue zone */
       else if (edge2 && edge2->blue_edge)
       {
         blue = edge2->blue_edge;
@@ -2348,7 +2454,7 @@ ta_latin_hint_edges(TA_GlyphHints hints,
     /* this should not happen, but it's better to be safe */
     if (edge2->blue_edge)
     {
-      TA_LOG(("  ASSERTION FAILED for edge %d\n", edge2-edges));
+      TA_LOG(("  ASSERTION FAILED for edge %d\n", edge2 - edges));
 
       ta_latin_align_linked_edge(hints, dim, edge2, edge);
       edge->flags |= TA_EDGE_DONE;
@@ -2809,6 +2915,8 @@ ta_latin_hints_apply(TA_GlyphHints hints,
   FT_Error error;
   int dim;
 
+  TA_LatinAxis  axis;
+
 
   error = ta_glyph_hints_reload(hints, outline);
   if (error)
@@ -2822,14 +2930,22 @@ ta_latin_hints_apply(TA_GlyphHints hints,
   if (TA_HINTS_DO_HORIZONTAL(hints))
 #endif
   {
-    error = ta_latin_hints_detect_features(hints, TA_DIMENSION_HORZ);
+    axis = &metrics->axis[TA_DIMENSION_HORZ];
+    error = ta_latin_hints_detect_features(hints,
+                                           axis->width_count,
+                                           axis->widths,
+                                           TA_DIMENSION_HORZ);
     if (error)
       goto Exit;
   }
 
   if (TA_HINTS_DO_VERTICAL(hints))
   {
-    error = ta_latin_hints_detect_features(hints, TA_DIMENSION_VERT);
+    axis = &metrics->axis[TA_DIMENSION_VERT];
+    error = ta_latin_hints_detect_features(hints,
+                                           axis->width_count,
+                                           axis->widths,
+                                           TA_DIMENSION_VERT);
     if (error)
       goto Exit;
 
