@@ -1,7 +1,7 @@
 /* taloader.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -120,7 +120,7 @@ ta_loader_load_g(TA_Loader loader,
   FT_Error error;
   FT_Face face = loader->face;
   TA_GlyphLoader gloader = loader->gloader;
-  TA_ScriptMetrics metrics = loader->metrics;
+  TA_StyleMetrics metrics = loader->metrics;
   TA_GlyphHints hints = &loader->hints;
   FT_GlyphSlot slot = face->glyph;
 #if 0
@@ -195,15 +195,15 @@ ta_loader_load_g(TA_Loader loader,
     /* now load the slot image into the auto-outline */
     /* and run the automatic hinting process */
     {
+      TA_StyleClass style_class = metrics->style_class;
       TA_WritingSystemClass writing_system_class =
-                              ta_writing_system_classes
-                                [metrics->script_class->writing_system];
+        ta_writing_system_classes[style_class->writing_system];
 
 
-      if (writing_system_class->script_hints_apply)
-        writing_system_class->script_hints_apply(hints,
-                                                 &gloader->current.outline,
-                                                 metrics);
+      if (writing_system_class->style_hints_apply)
+        writing_system_class->style_hints_apply(hints,
+                                                &gloader->current.outline,
+                                                metrics);
     }
 
     /* we now need to adjust the metrics according to the change in */
@@ -326,12 +326,7 @@ ta_loader_load_g(TA_Loader loader,
         /* recompute subglyph pointer */
         subglyph = gloader->base.subglyphs + num_base_subgs + nn;
 
-        if (subglyph->flags & FT_SUBGLYPH_FLAG_USE_MY_METRICS)
-        {
-          pp1 = loader->pp1;
-          pp2 = loader->pp2;
-        }
-        else
+        if (!(subglyph->flags & FT_SUBGLYPH_FLAG_USE_MY_METRICS))
         {
           loader->pp1 = pp1;
           loader->pp2 = pp2;
@@ -530,34 +525,34 @@ ta_loader_load_glyph(FONT* font,
   if (load_flags & (1 << 29))
     scaler.flags |= TA_SCALER_FLAG_NO_HORIZONTAL;
 
-  /* note that the fallback script can't be changed anymore */
+  /* note that the fallback style can't be changed anymore */
   /* after the first call of `ta_loader_load_glyph' */
   error = ta_loader_reset(font, face);
   if (!error)
   {
-    TA_ScriptMetrics metrics;
-    FT_UInt options = TA_SCRIPT_DFLT;
+    TA_StyleMetrics metrics;
+    FT_UInt options = TA_STYLE_NONE_DFLT;
 
 
 #ifdef FT_OPTION_AUTOFIT2
     /* XXX: undocumented hook to activate the latin2 hinter */
     if (load_flags & (1UL << 20))
-      options = TA_SCRIPT_LTN2;
+      options = TA_STYLE_LTN2_DFLT;
 #endif
 
     error = ta_face_globals_get_metrics(loader->globals, gindex,
                                         options, &metrics);
     if (!error)
     {
+      TA_StyleClass style_class = metrics->style_class;
       TA_WritingSystemClass writing_system_class =
-                              ta_writing_system_classes
-                                [metrics->script_class->writing_system];
+        ta_writing_system_classes[style_class->writing_system];
 
 
       loader->metrics = metrics;
 
-      if (writing_system_class->script_metrics_scale)
-        writing_system_class->script_metrics_scale(metrics, &scaler);
+      if (writing_system_class->style_metrics_scale)
+        writing_system_class->style_metrics_scale(metrics, &scaler);
       else
         metrics->scaler = scaler;
 
@@ -565,10 +560,10 @@ ta_loader_load_glyph(FONT* font,
                     | FT_LOAD_IGNORE_TRANSFORM;
       load_flags &= ~FT_LOAD_RENDER;
 
-      if (writing_system_class->script_hints_init)
+      if (writing_system_class->style_hints_init)
       {
-        error = writing_system_class->script_hints_init(&loader->hints,
-                                                        metrics);
+        error = writing_system_class->style_hints_init(&loader->hints,
+                                                       metrics);
         if (error)
           goto Exit;
       }
