@@ -1,7 +1,7 @@
 /* taloader.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -35,7 +35,7 @@ typedef struct TA_LoaderRec_
   /* current glyph data */
   TA_GlyphLoader gloader;
   TA_GlyphHintsRec hints;
-  TA_ScriptMetrics metrics;
+  TA_StyleMetrics metrics;
   FT_Bool transformed;
   FT_Matrix trans_matrix;
   FT_Vector trans_delta;
