@@ -1,7 +1,7 @@
 /* taprep.c */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -22,8 +22,11 @@
 unsigned char PREP(hinting_limit_a) [] =
 {
 
-  /* first of all, check whether we do hinting at all */
+  /* all our measurements are taken along the y axis, */
+  /* including the ppem and CVT values */
+  SVTCA_y,
 
+  /* first of all, check whether we do hinting at all */
   MPPEM,
   PUSHW_1,
 
@@ -44,14 +47,15 @@ unsigned char PREP(hinting_limit_b) [] =
 
 };
 
-/* we often need 0x10000 which can't be pushed directly onto the stack, */
-/* thus we provide it in the CVT as `cvtl_0x10000'; */
-/* at the same time, we store it in CVT index `cvtl_funits_to_pixels' also */
-/* as a scaled value to have a conversion factor from FUnits to pixels */
+/* we store 0x10000 in CVT index `cvtl_funits_to_pixels' as a scaled value */
+/* to have a conversion factor from FUnits to pixels */
 
-unsigned char PREP(store_0x10000) [] =
+unsigned char PREP(store_funits_to_pixels) [] =
 {
 
+  PUSHB_1,
+    cvtl_funits_to_pixels,
+
   PUSHW_2,
     0x08, /* 0x800 */
     0x00,
@@ -59,38 +63,10 @@ unsigned char PREP(store_0x10000) [] =
     0x00,
   MUL, /* 0x10000 */
 
-  DUP,
-  PUSHB_1,
-    cvtl_0x10000,
-  SWAP,
-  WCVTP,
-
-  PUSHB_1,
-    cvtl_funits_to_pixels,
-  SWAP,
   WCVTF, /* store value 1 in 16.16 format, scaled */
 
 };
 
-/* store number of scripts (needed in `fpgm' table) */
-
-unsigned char PREP(store_num_used_scripts_a) [] =
-{
-
-  PUSHB_2,
-    cvtl_num_used_scripts,
-
-};
-
-/*  %c, number of used scripts */
-
-unsigned char PREP(store_num_used_scripts_b) [] =
-{
-
-  WCVTP,
-
-};
-
 /* if the current ppem value is an exception, don't apply scaling */
 
 unsigned char PREP(test_exception_a) [] =
@@ -104,7 +80,7 @@ unsigned char PREP(test_exception_a) [] =
 
 };
 
-/* provide scaling factors for all scripts */
+/* provide scaling factors for all styles */
 
 unsigned char PREP(align_top_a) [] =
 {
@@ -116,11 +92,11 @@ unsigned char PREP(align_top_a) [] =
 
 };
 
-/*  PUSH (num_used_scripts + 2) */
+/*  PUSHB (num_used_styles + 2) */
 /*    ... */
-/*    %c, script 1's x height blue zone idx */
-/*    %c, script 0's x height blue zone idx */
-/*    %c, num_used_scripts */
+/*    %c, style 1's x height blue zone idx */
+/*    %c, style 0's x height blue zone idx */
+/*    %c, num_used_styles */
 
 unsigned char PREP(align_top_b) [] =
 {
@@ -133,7 +109,7 @@ unsigned char PREP(align_top_b) [] =
 unsigned char PREP(loop_cvt_a) [] =
 {
 
-    /* loop over (almost all) vertical CVT entries of all scripts, part 1 */
+    /* loop over (almost all) vertical CVT entries of all styles, part 1 */
     PUSHB_2,
       sal_i,
       CVT_SCALING_VALUE_OFFSET(0),
@@ -141,15 +117,15 @@ unsigned char PREP(loop_cvt_a) [] =
 
 };
 
-/*  PUSH (2*num_used_scripts + 2) */
+/*  PUSHB (2*num_used_styles + 2) */
 /*    ... */
-/*    %c, script 1's first vertical index */
-/*    %c, script 1's number of vertical indices */
+/*    %c, style 1's first vertical index */
+/*    %c, style 1's number of vertical indices */
 /*        (std. width, widths, flat blues zones without artifical ones) */
-/*    %c, script 0's first vertical index */
-/*    %c, script 0's number of vertical indices */
+/*    %c, style 0's first vertical index */
+/*    %c, style 0's number of vertical indices */
 /*        (std. width, widths, flat blues zones without artifical ones) */
-/*    %c, num_used_scripts */
+/*    %c, num_used_styles */
 
 unsigned char PREP(loop_cvt_b) [] =
 {
@@ -157,7 +133,7 @@ unsigned char PREP(loop_cvt_b) [] =
       bci_cvt_rescale_range,
     LOOPCALL,
 
-    /* loop over (almost all) vertical CVT entries of all scripts, part 2 */
+    /* loop over (almost all) vertical CVT entries of all styles, part 2 */
     PUSHB_2,
       sal_i,
       CVT_SCALING_VALUE_OFFSET(0),
@@ -165,13 +141,13 @@ unsigned char PREP(loop_cvt_b) [] =
 
 };
 
-/*  PUSH (2*num_used_scripts + 2) */
+/*  PUSHB (2*num_used_styles + 2) */
 /*    ... */
-/*    %c, script 1's first round blue zone index */
-/*    %c, script 1's number of round blue zones (without artificial ones) */
-/*    %c, script 0's first round blue zone index */
-/*    %c, script 0's number of round blue zones (without artificial ones) */
-/*    %c, num_used_scripts */
+/*    %c, style 1's first round blue zone index */
+/*    %c, style 1's number of round blue zones (without artificial ones) */
+/*    %c, style 0's first round blue zone index */
+/*    %c, style 0's number of round blue zones (without artificial ones) */
+/*    %c, num_used_styles */
 
 unsigned char PREP(loop_cvt_c) [] =
 {
@@ -205,16 +181,17 @@ unsigned char PREP(store_vwidth_data_b) [] =
 
 };
 
-/*PUSH (num_used_scripts + 2) */
+/*PUSHW (num_used_styles + 2) */
 /*  ... */
-/*  %c, script 1's first vertical width index */
-/*  %c, script 0's first vertical width index */
-/*  %c, num_used_scripts */
+/*  %d, style 1's first vertical width index (in multiples of 64) */
+/*  %d, style 0's first vertical width index (in multiples of 64) */
+/*  %d, num_used_styles */
 
 unsigned char PREP(store_vwidth_data_c) [] =
 {
 
-    bci_vwidth_data_store,
+    0x00, /* high byte */
+    bci_vwidth_data_store, /* low byte */
   LOOPCALL,
 
   PUSHB_2,
@@ -231,37 +208,22 @@ unsigned char PREP(store_vwidth_data_d) [] =
 
 };
 
-/*PUSH (num_used_scripts + 2) */
+/*PUSHW (num_used_styles + 2) */
 /*  ... */
-/*  %c, script 1's number of vertical widths */
-/*  %c, script 0's number of vertical widths */
-/*  %c, num_used_scripts */
+/*  %d, style 1's number of vertical widths (in multiples of 64) */
+/*  %d, style 0's number of vertical widths (in multiples of 64) */
+/*  %d, num_used_styles */
 
 unsigned char PREP(store_vwidth_data_e) [] =
 {
 
-    bci_vwidth_data_store,
+    0x00, /* high byte */
+    bci_vwidth_data_store, /* low byte */
   LOOPCALL,
 
 };
 
-/*PUSH (2*num_used_scripts + 2) */
-/*  ... */
-/*  %c, script 1's first blue ref index */
-/*  %c, script 1's number of blue ref indices */
-/*  %c, script 0's first blue ref index */
-/*  %c, script 0's number of blue ref indices */
-/*  %c, num_used_scripts */
-
-unsigned char PREP(round_blues) [] =
-{
-
-    bci_blue_round_range,
-  LOOPCALL,
-
-};
-
-unsigned char PREP(set_stem_width_handling_a) [] =
+unsigned char PREP(set_smooth_or_strong_a) [] =
 {
 
   /*
@@ -276,19 +238,20 @@ unsigned char PREP(set_stem_width_handling_a) [] =
    * this works for TrueType version >= 38), checking whether sub-pixel
    * positioning is available.
    *
-   * If GDI ClearType is active, we use a different stem width function
-   * which snaps to integer pixels as much as possible.
+   * If GDI ClearType is active, we use different functions for stem width
+   * computation and blue zone rounding that snap to integer pixels as much
+   * as possible.
    */
 
-  /* set default positioning */
+  /* set default value */
   PUSHB_2,
-    cvtl_stem_width_function,
+    cvtl_use_strong_functions,
 
 };
 
-/*  %d, either bci_smooth_stem_width or bci_strong_stem_width */
+/*  %c, either 0 or 100 */
 
-unsigned char PREP(set_stem_width_handling_b) [] =
+unsigned char PREP(set_smooth_or_strong_b) [] =
 {
 
   WCVTP,
@@ -309,12 +272,12 @@ unsigned char PREP(set_stem_width_handling_b) [] =
     GETINFO,
     IF,
       PUSHB_2,
-        cvtl_stem_width_function,
+        cvtl_use_strong_functions,
 };
 
-/*      %d, either bci_smooth_stem_width or bci_strong_stem_width */
+/*      %c, either 0 or 100 */
 
-unsigned char PREP(set_stem_width_handling_c) [] =
+unsigned char PREP(set_smooth_or_strong_c) [] =
 {
 
       WCVTP,
@@ -345,13 +308,13 @@ unsigned char PREP(set_stem_width_handling_c) [] =
         EQ,
         IF,
           PUSHB_2,
-            cvtl_stem_width_function,
+            cvtl_use_strong_functions,
 
 };
 
-/*          %d, either bci_smooth_stem_width or bci_strong_stem_width */
+/*          %c, either 0 or 100 */
 
-unsigned char PREP(set_stem_width_handling_d) [] =
+unsigned char PREP(set_smooth_or_strong_d) [] =
 {
 
           WCVTP,
@@ -362,6 +325,22 @@ unsigned char PREP(set_stem_width_handling_d) [] =
 
 };
 
+/*PUSHB (2*num_used_styles + 2) */
+/*  ... */
+/*  %c, style 1's first blue ref index */
+/*  %c, style 1's number of blue ref indices */
+/*  %c, style 0's first blue ref index */
+/*  %c, style 0's number of blue ref indices */
+/*  %c, num_used_styles */
+
+unsigned char PREP(round_blues) [] =
+{
+
+    bci_blue_round_range,
+  LOOPCALL,
+
+};
+
 unsigned char PREP(set_dropout_mode) [] =
 {
 
@@ -593,27 +572,24 @@ TA_table_build_prep(FT_Byte** prep,
                    + 2
                    + sizeof (PREP(hinting_limit_b));
 
-  buf_new_len += sizeof (PREP(store_0x10000));
-  buf_new_len += sizeof (PREP(store_num_used_scripts_a))
-                 + 1
-                 + sizeof (PREP(store_num_used_scripts_b));
+  buf_new_len += sizeof (PREP(store_funits_to_pixels));
 
   if (font->x_height_snapping_exceptions)
     buf_new_len += sizeof (PREP(test_exception_a));
 
   buf_new_len += sizeof (PREP(align_top_a))
-                 + (data->num_used_scripts > 6
-                      ? data->num_used_scripts + 3
-                      : data->num_used_scripts + 2)
+                 + (data->num_used_styles > 6
+                      ? data->num_used_styles + 3
+                      : data->num_used_styles + 2)
                  + sizeof (PREP(align_top_b));
   buf_new_len += sizeof (PREP(loop_cvt_a))
-                 + (data->num_used_scripts > 3
-                     ? 2 * data->num_used_scripts + 3
-                     : 2 * data->num_used_scripts + 2)
+                 + (data->num_used_styles > 3
+                     ? 2 * data->num_used_styles + 3
+                     : 2 * data->num_used_styles + 2)
                  + sizeof (PREP(loop_cvt_b))
-                 + (data->num_used_scripts > 3
-                     ? 2 * data->num_used_scripts + 3
-                     : 2 * data->num_used_scripts + 2)
+                 + (data->num_used_styles > 3
+                     ? 2 * data->num_used_styles + 3
+                     : 2 * data->num_used_styles + 2)
                  + sizeof (PREP(loop_cvt_c));
 
   if (font->x_height_snapping_exceptions)
@@ -622,27 +598,27 @@ TA_table_build_prep(FT_Byte** prep,
   buf_new_len += sizeof (PREP(store_vwidth_data_a))
                  + 1
                  + sizeof (PREP(store_vwidth_data_b))
-                 + (data->num_used_scripts > 6
-                     ? data->num_used_scripts + 3
-                     : data->num_used_scripts + 2)
+                 + (data->num_used_styles > 6
+                      ? 2 * (data->num_used_styles + 1) + 2
+                      : 2 * (data->num_used_styles + 1) + 1)
                  + sizeof (PREP(store_vwidth_data_c))
                  + 1
                  + sizeof (PREP(store_vwidth_data_d))
-                 + (data->num_used_scripts > 6
-                     ? data->num_used_scripts + 3
-                     : data->num_used_scripts + 2)
+                 + (data->num_used_styles > 6
+                      ? 2 * (data->num_used_styles + 1) + 2
+                      : 2 * (data->num_used_styles + 1) + 1)
                  + sizeof (PREP(store_vwidth_data_e));
-  buf_new_len += (data->num_used_scripts > 3
-                     ? 2 * data->num_used_scripts + 3
-                     : 2 * data->num_used_scripts + 2)
+  buf_new_len += sizeof (PREP(set_smooth_or_strong_a))
+                 + 1
+                 + sizeof (PREP(set_smooth_or_strong_b))
+                 + 1
+                 + sizeof (PREP(set_smooth_or_strong_c))
+                 + 1
+                 + sizeof (PREP(set_smooth_or_strong_d));
+  buf_new_len += (data->num_used_styles > 3
+                     ? 2 * data->num_used_styles + 3
+                     : 2 * data->num_used_styles + 2)
                  + sizeof (PREP(round_blues));
-  buf_new_len += sizeof (PREP(set_stem_width_handling_a))
-                 + 1
-                 + sizeof (PREP(set_stem_width_handling_b))
-                 + 1
-                 + sizeof (PREP(set_stem_width_handling_c))
-                 + 1
-                 + sizeof (PREP(set_stem_width_handling_d));
   buf_new_len += sizeof (PREP(set_dropout_mode));
   buf_new_len += sizeof (PREP(reset_component_counter));
 
@@ -673,76 +649,74 @@ TA_table_build_prep(FT_Byte** prep,
     COPY_PREP(hinting_limit_b);
   }
 
-  COPY_PREP(store_0x10000);
-
-  COPY_PREP(store_num_used_scripts_a);
-  *(bufp++) = (unsigned char)data->num_used_scripts;
-  COPY_PREP(store_num_used_scripts_b);
+  COPY_PREP(store_funits_to_pixels);
 
   if (font->x_height_snapping_exceptions)
     COPY_PREP(test_exception_a);
 
   COPY_PREP(align_top_a);
-  if (data->num_used_scripts > 6)
+  if (data->num_used_styles > 6)
   {
     BCI(NPUSHB);
-    BCI(data->num_used_scripts + 2);
+    BCI(data->num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + data->num_used_scripts + 2);
+    BCI(PUSHB_1 - 1 + data->num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
-  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
-    if (data->script_ids[i] == 0xFFFFU)
+    if (data->style_ids[i] == 0xFFFFU)
       continue;
 
     *(bufp++) = CVT_X_HEIGHT_BLUE_OFFSET(i) >= 0xFFFFU
                   ? 0
                   : (unsigned char)CVT_X_HEIGHT_BLUE_OFFSET(i);
   }
-  *(bufp++) = data->num_used_scripts;
+  *(bufp++) = data->num_used_styles;
   COPY_PREP(align_top_b);
 
   COPY_PREP(loop_cvt_a);
-  if (data->num_used_scripts > 3)
+  if (data->num_used_styles > 3)
   {
     BCI(NPUSHB);
-    BCI(2 * data->num_used_scripts + 2);
+    BCI(2 * data->num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + 2 * data->num_used_scripts + 2);
+    BCI(PUSHB_1 - 1 + 2 * data->num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
-  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
-    if (data->script_ids[i] == 0xFFFFU)
+    if (data->style_ids[i] == 0xFFFFU)
       continue;
 
     /* don't loop over artificial blue zones */
     *(bufp++) = (unsigned char)CVT_VERT_STANDARD_WIDTH_OFFSET(i);
-    *(bufp++) = (unsigned char)(1
-                                + CVT_VERT_WIDTHS_SIZE(i)
-                                + CVT_BLUES_SIZE(i) - 2);
+    *(bufp++) = (unsigned char)(
+                  1
+                  + CVT_VERT_WIDTHS_SIZE(i)
+                  + (CVT_BLUES_SIZE(i) > 1 ? CVT_BLUES_SIZE(i) - 2 : 0));
   }
-  *(bufp++) = data->num_used_scripts;
+  *(bufp++) = data->num_used_styles;
   COPY_PREP(loop_cvt_b);
-  if (data->num_used_scripts > 3)
+  if (data->num_used_styles > 3)
   {
     BCI(NPUSHB);
-    BCI(2 * data->num_used_scripts + 2);
+    BCI(2 * data->num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + 2 * data->num_used_scripts + 2);
+    BCI(PUSHB_1 - 1 + 2 * data->num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
-  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
-    if (data->script_ids[i] == 0xFFFFU)
+    if (data->style_ids[i] == 0xFFFFU)
       continue;
 
     /* don't loop over artificial blue zones */
     *(bufp++) = (unsigned char)CVT_BLUE_SHOOTS_OFFSET(i);
-    *(bufp++) = (unsigned char)(CVT_BLUES_SIZE(i) - 2);
+    *(bufp++) = (unsigned char)(
+                  CVT_BLUES_SIZE(i) > 1 ? CVT_BLUES_SIZE(i) - 2 : 0);
   }
-  *(bufp++) = data->num_used_scripts;
+  *(bufp++) = data->num_used_styles;
   COPY_PREP(loop_cvt_c);
 
   if (font->x_height_snapping_exceptions)
@@ -751,72 +725,72 @@ TA_table_build_prep(FT_Byte** prep,
   COPY_PREP(store_vwidth_data_a);
   *(bufp++) = (unsigned char)CVT_VWIDTH_OFFSET_DATA(0);
   COPY_PREP(store_vwidth_data_b);
-  if (data->num_used_scripts > 6)
+  if (data->num_used_styles > 6)
   {
-    BCI(NPUSHB);
-    BCI(data->num_used_scripts + 2);
+    BCI(NPUSHW);
+    BCI(data->num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + data->num_used_scripts + 2);
-  /* XXX: make this work for offsets > 255 */
-  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+    BCI(PUSHW_1 - 1 + data->num_used_styles + 2);
+  for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
-    if (data->script_ids[i] == 0xFFFFU)
+    if (data->style_ids[i] == 0xFFFFU)
       continue;
 
-    *(bufp++) = (unsigned char)CVT_VERT_WIDTHS_OFFSET(i);
+    *(bufp++) = HIGH(CVT_VERT_WIDTHS_OFFSET(i) * 64);
+    *(bufp++) = LOW(CVT_VERT_WIDTHS_OFFSET(i) * 64);
   }
-  *(bufp++) = data->num_used_scripts;
+  *(bufp++) = HIGH(data->num_used_styles);
+  *(bufp++) = LOW(data->num_used_styles);
   COPY_PREP(store_vwidth_data_c);
   *(bufp++) = (unsigned char)CVT_VWIDTH_SIZE_DATA(0);
   COPY_PREP(store_vwidth_data_d);
-  if (data->num_used_scripts > 6)
+  if (data->num_used_styles > 6)
   {
-    BCI(NPUSHB);
-    BCI(data->num_used_scripts + 2);
+    BCI(NPUSHW);
+    BCI(data->num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + data->num_used_scripts + 2);
-  /* XXX: make this work for values > 255 */
-  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+    BCI(PUSHW_1 - 1 + data->num_used_styles + 2);
+  for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
-    if (data->script_ids[i] == 0xFFFFU)
+    if (data->style_ids[i] == 0xFFFFU)
       continue;
 
-    *(bufp++) = (unsigned char)CVT_VERT_WIDTHS_SIZE(i);
+    *(bufp++) = HIGH(CVT_VERT_WIDTHS_SIZE(i) * 64);
+    *(bufp++) = LOW(CVT_VERT_WIDTHS_SIZE(i) * 64);
   }
-  *(bufp++) = data->num_used_scripts;
+  *(bufp++) = HIGH(data->num_used_styles);
+  *(bufp++) = LOW(data->num_used_styles);
   COPY_PREP(store_vwidth_data_e);
 
-  if (data->num_used_scripts > 3)
+  COPY_PREP(set_smooth_or_strong_a);
+  *(bufp++) = font->gray_strong_stem_width ? 100 : 0;
+  COPY_PREP(set_smooth_or_strong_b);
+  *(bufp++) = font->gdi_cleartype_strong_stem_width ? 100 : 0;
+  COPY_PREP(set_smooth_or_strong_c);
+  *(bufp++) = font->dw_cleartype_strong_stem_width ? 100 : 0;
+  COPY_PREP(set_smooth_or_strong_d);
+
+  if (data->num_used_styles > 3)
   {
     BCI(NPUSHB);
-    BCI(2 * data->num_used_scripts + 2);
+    BCI(2 * data->num_used_styles + 2);
   }
   else
-    BCI(PUSHB_1 - 1 + 2 * data->num_used_scripts + 2);
+    BCI(PUSHB_1 - 1 + 2 * data->num_used_styles + 2);
   /* XXX: make this work for offsets > 255 */
-  for (i = TA_SCRIPT_MAX - 1; i >= 0; i--)
+  for (i = TA_STYLE_MAX - 1; i >= 0; i--)
   {
-    if (data->script_ids[i] == 0xFFFFU)
+    if (data->style_ids[i] == 0xFFFFU)
       continue;
 
     *(bufp++) = (unsigned char)CVT_BLUE_REFS_OFFSET(i);
     *(bufp++) = (unsigned char)CVT_BLUES_SIZE(i);
   }
-  *(bufp++) = data->num_used_scripts;
+  *(bufp++) = data->num_used_styles;
   COPY_PREP(round_blues);
 
-  COPY_PREP(set_stem_width_handling_a);
-  *(bufp++) = font->gray_strong_stem_width ? bci_strong_stem_width
-                                           : bci_smooth_stem_width;
-  COPY_PREP(set_stem_width_handling_b);
-  *(bufp++) = font->gdi_cleartype_strong_stem_width ? bci_strong_stem_width
-                                                    : bci_smooth_stem_width;
-  COPY_PREP(set_stem_width_handling_c);
-  *(bufp++) = font->dw_cleartype_strong_stem_width ? bci_strong_stem_width
-                                                   : bci_smooth_stem_width;
-  COPY_PREP(set_stem_width_handling_d);
   COPY_PREP(set_dropout_mode);
   COPY_PREP(reset_component_counter);
 
