@@ -1,7 +1,7 @@
 /* tatypes.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -27,6 +27,10 @@
 
 #include "tablue.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 
 /* enable one of the following three definitions for debugging */
 /* #define TA_DEBUG */
@@ -158,59 +162,44 @@ typedef struct TA_ScalerRec_
            && (a)->y_delta == (b)->y_delta)
 
 
-/* This is the main structure which combines writing systems and script */
-/* data (for a given face object, see below). */
-
-typedef struct TA_WritingSystemClassRec_ const* TA_WritingSystemClass;
-typedef struct TA_ScriptClassRec_ const* TA_ScriptClass;
-typedef struct TA_FaceGlobalsRec_* TA_FaceGlobals;
-
-typedef struct TA_ScriptMetricsRec_
-{
-  TA_ScriptClass script_class;
-  TA_ScalerRec scaler;
-  FT_Bool digits_have_same_width;
-
-  TA_FaceGlobals globals; /* to access properties */
-} TA_ScriptMetricsRec, *TA_ScriptMetrics;
-
+typedef struct TA_StyleMetricsRec_* TA_StyleMetrics;
 
 /* this function parses an FT_Face to compute global metrics */
-/* for a specific script */
+/* for a specific style */
 typedef FT_Error
-(*TA_Script_InitMetricsFunc)(TA_ScriptMetrics metrics,
-                             FT_Face face);
+(*TA_WritingSystem_InitMetricsFunc)(TA_StyleMetrics metrics,
+                                    FT_Face face);
 typedef void
-(*TA_Script_ScaleMetricsFunc)(TA_ScriptMetrics metrics,
-                              TA_Scaler scaler);
+(*TA_WritingSystem_ScaleMetricsFunc)(TA_StyleMetrics metrics,
+                                     TA_Scaler scaler);
 typedef void
-(*TA_Script_DoneMetricsFunc)(TA_ScriptMetrics metrics);
+(*TA_WritingSystem_DoneMetricsFunc)(TA_StyleMetrics metrics);
 typedef FT_Error
-(*TA_Script_InitHintsFunc)(TA_GlyphHints hints,
-                           TA_ScriptMetrics metrics);
+(*TA_WritingSystem_InitHintsFunc)(TA_GlyphHints hints,
+                                  TA_StyleMetrics metrics);
 typedef void
-(*TA_Script_ApplyHintsFunc)(TA_GlyphHints hints,
-                            FT_Outline* outline,
-                            TA_ScriptMetrics metrics);
+(*TA_WritingSystem_ApplyHintsFunc)(TA_GlyphHints hints,
+                                   FT_Outline* outline,
+                                   TA_StyleMetrics metrics);
 
 
 /*
- *  In FreeType, a writing system consists of multiple scripts which can
- *  be handled similarly *in a typographical way*; the relationship is not
- *  based on history.  For example, both the Greek and the unrelated
- *  Armenian scripts share the same features like ascender, descender,
- *  x-height, etc.  Essentially, a writing system is covered by a
- *  submodule of the auto-fitter; it contains
+ * For the auto-hinter, a writing system consists of multiple scripts that
+ * can be handled similarly *in a typographical way*; the relationship is
+ * not based on history.  For example, both the Greek and the unrelated
+ * Armenian scripts share the same features like ascender, descender,
+ * x-height, etc.  Essentially, a writing system is covered by a
+ * submodule of the auto-fitter; it contains
  *
- *  - a specific global analyzer which computes global metrics specific to
- *    the script (based on script-specific characters to identify ascender
- *    height, x-height, etc.),
+ * - a specific global analyzer that computes global metrics specific to
+ *   the script (based on script-specific characters to identify ascender
+ *   height, x-height, etc.),
  *
- *  - a specific glyph analyzer that computes segments and edges for each
- *    glyph covered by the script,
+ * - a specific glyph analyzer that computes segments and edges for each
+ *   glyph covered by the script,
  *
- *  - a specific grid-fitting algorithm that distorts the scaled glyph
- *    outline according to the results of the glyph analyzer.
+ * - a specific grid-fitting algorithm that distorts the scaled glyph
+ *   outline according to the results of the glyph analyzer.
  */
 
 #define __TAWRTSYS_H__ /* don't load header files */
@@ -233,26 +222,28 @@ typedef struct TA_WritingSystemClassRec_
 {
   TA_WritingSystem writing_system;
 
-  FT_Offset script_metrics_size;
-  TA_Script_InitMetricsFunc script_metrics_init;
-  TA_Script_ScaleMetricsFunc script_metrics_scale;
-  TA_Script_DoneMetricsFunc script_metrics_done;
+  FT_Offset style_metrics_size;
+  TA_WritingSystem_InitMetricsFunc style_metrics_init;
+  TA_WritingSystem_ScaleMetricsFunc style_metrics_scale;
+  TA_WritingSystem_DoneMetricsFunc style_metrics_done;
 
-  TA_Script_InitHintsFunc script_hints_init;
-  TA_Script_ApplyHintsFunc script_hints_apply;
+  TA_WritingSystem_InitHintsFunc style_hints_init;
+  TA_WritingSystem_ApplyHintsFunc style_hints_apply;
 } TA_WritingSystemClassRec;
 
+typedef const TA_WritingSystemClassRec* TA_WritingSystemClass;
+
 
 /*
- *  Each script is associated with a set of Unicode ranges which gets used
- *  to test whether the font face supports the script.  It also references
- *  the writing system it belongs to.
+ * Each script is associated with a set of Unicode ranges that gets used
+ * to test whether the font face supports the script.
  *
- *  We use four-letter script tags from the OpenType specification.
+ * We use four-letter script tags from the OpenType specification,
+ * extended by `NONE', which indicates `no script'.
  */
 
 #undef SCRIPT
-#define SCRIPT(s, S, d) \
+#define SCRIPT(s, S, d, h, sc1, sc2, sc3) \
           TA_SCRIPT_ ## S,
 
 /* The list of known scripts. */
@@ -278,13 +269,115 @@ typedef const TA_Script_UniRangeRec* TA_Script_UniRange;
 typedef struct TA_ScriptClassRec_
 {
   TA_Script script;
-  TA_Blue_Stringset blue_stringset;
-  TA_WritingSystem writing_system;
 
   TA_Script_UniRange script_uni_ranges; /* last must be { 0, 0 } */
-  FT_UInt32 standard_char; /* for default width and height */
+
+  FT_UInt32 standard_char1; /* for default width and height */
+  FT_UInt32 standard_char2; /* ditto */
+  FT_UInt32 standard_char3; /* ditto */
 } TA_ScriptClassRec;
 
+typedef const TA_ScriptClassRec*  TA_ScriptClass;
+
+
+/*
+ * Usually, a font contains more glyphs than can be addressed by its
+ * character map.
+ *
+ * In the PostScript font world, encoding vectors specific to a given
+ * task are used to select such glyphs, and these glyphs can be often
+ * recognized by having a suffix in its glyph names.  For example, a
+ * superscript glyph `A' might be called `A.sup'.  Unfortunately, this
+ * naming scheme is not standardized and thus unusable for us.
+ *
+ * In the OpenType world, a better solution was invented, namely
+ * `features', which cleanly separate a character's input encoding from
+ * the corresponding glyph's appearance, and which don't use glyph names
+ * at all.  For our purposes, and slightly generalized, an OpenType
+ * feature is a name of a mapping that maps character codes to
+ * non-standard glyph indices (features get used for other things also).
+ * For example, the `sups' feature provides superscript glyphs, thus
+ * mapping character codes like `A' or `B' to superscript glyph
+ * representation forms.  How this mapping happens is completely
+ * uninteresting to us.
+ *
+ * For the auto-hinter, a `coverage' represents all glyphs of an OpenType
+ * feature collected in a set (as listed below) that can be hinted
+ * together.  To continue the above example, superscript glyphs must not
+ * be hinted together with normal glyphs because the blue zones
+ * completely differ.
+ *
+ * To compute coverages, we use the HarfBuzz library, which has many
+ * functions exactly for this purpose.
+ *
+ * TA_COVERAGE_DEFAULT is special: It should cover everything that isn't
+ * listed separately (including the glyphs addressable by the character
+ * map).
+ */
+
+#undef COVERAGE
+#define COVERAGE(name, NAME, description, \
+                 tag1, tag2, tag3, tag4) \
+          TA_COVERAGE_ ## NAME,
+
+typedef enum TA_Coverage_
+{
+#include "tacover.h"
+
+  TA_COVERAGE_DEFAULT
+} TA_Coverage;
+
+
+/*
+ * The topmost structure for modelling the auto-hinter glyph input data
+ * is a `style class', grouping everything together.
+ */
+
+#undef STYLE
+#define STYLE(s, S, d, ws, sc, ss, c) \
+          TA_STYLE_ ## S,
+
+/* The list of known styles. */
+typedef enum  TA_Style_
+{
+#include "tastyles.h"
+
+  TA_STYLE_MAX /* do not remove */
+} TA_Style;
+
+
+typedef struct TA_StyleClassRec_
+{
+  TA_Style style;
+
+  TA_WritingSystem writing_system;
+  TA_Script script;
+  TA_Blue_Stringset blue_stringset;
+  TA_Coverage coverage;
+} TA_StyleClassRec;
+
+typedef const TA_StyleClassRec* TA_StyleClass;
+
+
+typedef struct TA_FaceGlobalsRec_*  TA_FaceGlobals;
+
+/* This is the main structure that combines everything.  Autofit modules */
+/* specific to writing systems derive their structures from it, for      */
+/* example `TA_LatinMetrics'.                                            */
+
+typedef struct  TA_StyleMetricsRec_
+{
+  TA_StyleClass style_class;
+  TA_ScalerRec scaler;
+  FT_Bool digits_have_same_width;
+
+  TA_FaceGlobals globals; /* to access properties */
+} TA_StyleMetricsRec;
+
+#ifdef __cplusplus
+} /* extern "C" */
+#endif
+
 #endif /* __TATYPES_H__ */
 
 /* end of tatypes.h */
