@@ -1,7 +1,7 @@
 /* ttfautohint-errors.h */
 
 /*
- * Copyright (C) 2011-2013 by Werner Lemberg.
+ * Copyright (C) 2011-2014 by Werner Lemberg.
  *
  * This file is part of the ttfautohint library, and may only be used,
  * modified, and distributed under the terms given in `COPYING'.  By
@@ -56,7 +56,7 @@ TA_ERRORDEF_(Invalid_Stream_Write,     0x5F, \
 TA_ERRORDEF_(Hinter_Overflow,          0xF0, \
              "hinter overflow")
 TA_ERRORDEF_(Missing_Glyph,            0xF1, \
-             "missing key character glyph")
+             "missing standard character glyph")
 TA_ERRORDEF_(Missing_Unicode_CMap,     0xF2, \
              "missing Unicode character map")
 TA_ERRORDEF_(Missing_Symbol_CMap,      0xF3, \
